test_name "Dashboard Rake Tasks (legacy syntax)"

dashboard_hosts = hosts.select { |h| h['roles'].include? 'dashboard' }

step "Verify there's at least one dashboard host" do
  assert(dashboard_hosts.length > 0, "No dashboard hosts found!")
end

def build_rake_cmd_string(host, cmd)
  rake_path = host.is_pe? ? "/opt/puppet/bin/rake" : "rake"
  rakefile_path = host.is_pe? ? "/opt/puppet/share/puppet-dashboard/Rakefile" : "/usr/share/puppet-dashboard/Rakefile"
  "RAILS_ENV=production #{rake_path} -f #{rakefile_path} #{cmd}"
end

def run_rake_task(host, cmd)
  result = on(host, build_rake_cmd_string(host, cmd))
  result.stdout.split("\n")
end


#################################
# `nodeclass` tasks
#################################

step "test nodeclass:add, nodeclass:list, nodeclass:del rake tasks" do
  dashboard_hosts.each do |dashboard|
    run_rake_task(dashboard, "nodeclass:add name=fooclass")
    results = run_rake_task(dashboard, "nodeclass:list")
    assert(results.include?("fooclass"), "nodeclass:add should have added 'fooclass'!  nodeclass:list returned: #{results.join("\n")}")

    run_rake_task(dashboard, "nodeclass:del name=fooclass")
    results = run_rake_task(dashboard, "nodeclass:list")
    assert((! results.include?("fooclass")), "nodeclass:del should have removed 'fooclass'!  nodeclass:list returned: #{results.join("\n")}")
  end
end

#################################
# `nodegroup` tasks
#################################

step "test nodegroup:add, nodegroup:list, nodegroup:del rake tasks" do
  dashboard_hosts.each do |dashboard|
    run_rake_task(dashboard, "nodegroup:add name=foogroup")
    results = run_rake_task(dashboard, "nodegroup:list")
    assert(results.include?("foogroup"), "nodegroup:add should have added 'foogroup'!  nodegroup:list returned: #{results.join("\n")}")

    run_rake_task(dashboard, "nodegroup:del name=foogroup")
    results = run_rake_task(dashboard, "nodegroup:list")
    assert((! results.include?("foogroup")), "nodegroup:del should have removed 'foogroup'!  nodegroup:list returned: #{results.join("\n")}")
  end
end


step "test nodegroup:addclass, nodegroup:listclasses, nodegroup:delclass rake tasks" do
  dashboard_hosts.each do |dashboard|
    run_rake_task(dashboard, "nodegroup:add name=foogroup")
    run_rake_task(dashboard, "nodeclass:add name=fooclass")
    run_rake_task(dashboard, "nodeclass:add name=barclass")

    run_rake_task(dashboard, "nodegroup:addclass name=foogroup class=fooclass")
    run_rake_task(dashboard, "nodegroup:addclass name=foogroup class=barclass")

    results = run_rake_task(dashboard, "nodegroup:listclasses name=foogroup")
    assert(results.include?("fooclass"), "nodegroup:addclass should have added 'fooclass'!  nodegroup:listclasses returned: #{results.join("\n")}")
    assert(results.include?("barclass"), "nodegroup:addclass should have added 'barclass'!  nodegroup:listclasses returned: #{results.join("\n")}")

    run_rake_task(dashboard, "nodegroup:delclass name=foogroup class=fooclass")
    run_rake_task(dashboard, "nodegroup:delclass name=foogroup class=barclass")

    results = run_rake_task(dashboard, "nodegroup:listclasses name=foogroup")
    assert((! results.include?("fooclass")), "nodegroup:delclass should have removed 'fooclass'!  nodegroup:listclasses returned: #{results.join("\n")}")
    assert((! results.include?("barclass")), "nodegroup:delclass should have removed 'barclass'!  nodegroup:listclasses returned: #{results.join("\n")}")

    run_rake_task(dashboard, "nodegroup:del name=foogroup")
    run_rake_task(dashboard, "nodeclass:del name=fooclass")
    run_rake_task(dashboard, "nodeclass:del name=barclass")
  end
end


step "test nodegroup:edit rake task" do
  dashboard_hosts.each do |dashboard|
    run_rake_task(dashboard, "nodegroup:add name=foogroup")
    run_rake_task(dashboard, "nodeclass:add name=fooclass")
    run_rake_task(dashboard, "nodeclass:add name=barclass")

    run_rake_task(dashboard, "nodegroup:edit name=foogroup classes=fooclass,barclass")

    results = run_rake_task(dashboard, "nodegroup:listclasses name=foogroup")
    assert(results.include?("fooclass"), "nodegroup:edit should have added 'fooclass'!  nodegroup:listclasses returned: #{results.join("\n")}")
    assert(results.include?("barclass"), "nodegroup:edit should have added 'barclass'!  nodegroup:listclasses returned: #{results.join("\n")}")

    run_rake_task(dashboard, "nodegroup:edit name=foogroup classes=\"\"")

    results = run_rake_task(dashboard, "nodegroup:listclasses name=foogroup")
    assert((! results.include?("fooclass")), "nodegroup:edit should have removed 'fooclass'!  nodegroup:listclasses returned: #{results.join("\n")}")
    assert((! results.include?("barclass")), "nodegroup:edit should have removed 'barclass'!  nodegroup:listclasses returned: #{results.join("\n")}")

    run_rake_task(dashboard, "nodegroup:del name=foogroup")
    run_rake_task(dashboard, "nodeclass:del name=fooclass")
    run_rake_task(dashboard, "nodeclass:del name=barclass")
  end
end


step "test nodegroup:add_all_nodes rake task with deprecated 'group' env variable" do
  dashboard_hosts.each do |dashboard|
    run_rake_task(dashboard, "nodegroup:add name=foogroup")
    run_rake_task(dashboard, "node:add name=foo.localdomain")
    run_rake_task(dashboard, "node:add name=bar.localdomain")

    # "group" is deprecated, in favour of "name"
    results = run_rake_task(dashboard, "nodegroup:add_all_nodes group=foogroup")
    assert(results.include?('Please use "name" to specify the group name instead.'),
           "nodegroup:add_all_nodes should have shown a deprecation warning when using \"group\" to specify the group name")

    results = run_rake_task(dashboard, "node:listgroups name=foo.localdomain")
    assert(results.include?("foogroup"), "nodegroup:add_all_nodes should have added 'foogroup'!  node:listgroups returned: #{results.join("\n")}")

    results = run_rake_task(dashboard, "node:listgroups name=bar.localdomain")
    assert(results.include?("foogroup"), "nodegroup:add_all_nodes should have added 'foogroup'!  node:listgroups returned: #{results.join("\n")}")

    run_rake_task(dashboard, "node:del name=foo.localdomain")
    run_rake_task(dashboard, "node:del name=bar.localdomain")
    run_rake_task(dashboard, "nodegroup:del name=foogroup")
  end
end


step "test nodegroup:add_all_nodes rake task" do
  dashboard_hosts.each do |dashboard|
    run_rake_task(dashboard, "nodegroup:add name=foogroup")
    run_rake_task(dashboard, "node:add name=foo.localdomain")
    run_rake_task(dashboard, "node:add name=bar.localdomain")

    run_rake_task(dashboard, "nodegroup:add_all_nodes name=foogroup")

    results = run_rake_task(dashboard, "node:listgroups name=foo.localdomain")
    assert(results.include?("foogroup"), "nodegroup:add_all_nodes should have added 'foogroup'!  node:listgroups returned: #{results.join("\n")}")

    results = run_rake_task(dashboard, "node:listgroups name=bar.localdomain")
    assert(results.include?("foogroup"), "nodegroup:add_all_nodes should have added 'foogroup'!  node:listgroups returned: #{results.join("\n")}")

    run_rake_task(dashboard, "node:del name=foo.localdomain")
    run_rake_task(dashboard, "node:del name=bar.localdomain")
    run_rake_task(dashboard, "nodegroup:del name=foogroup")
  end
end


step "test nodegroup:addgroup, nodegroup:listgroups, nodegroup:delgroup" do
  dashboard_hosts.each do |dashboard|
    run_rake_task(dashboard, "nodegroup:add name=parentgroup")
    run_rake_task(dashboard, "nodegroup:add name=childgroup")

    run_rake_task(dashboard, "nodegroup:addgroup name=parentgroup group=childgroup")

    results = run_rake_task(dashboard, "nodegroup:listgroups name=parentgroup")
    assert(results.include?("childgroup"), "nodegroup:addgroup should have added 'childgroup'!  nodegroup:listgroups returned: #{results.join("\n")}")

    run_rake_task(dashboard, "nodegroup:delgroup name=parentgroup group=childgroup")

    results = run_rake_task(dashboard, "nodegroup:listgroups name=parentgroup")
    assert(!results.include?("childgroup"), "nodegroup:delgroup should have removed 'childgroup'!  nodegroup:listgroups returned: #{results.join("\n")}")

    run_rake_task(dashboard, "nodegroup:del name=parentgroup")
    run_rake_task(dashboard, "nodegroup:del name=childgroup")
  end
end


step "test nodegroup:addclassparam, nodegroup:delclassparam, nodegroup:listclassparams rake tasks" do
  dashboard_hosts.each do |dashboard|
    run_rake_task(dashboard, "nodegroup:add name=foogroup")
    run_rake_task(dashboard, "nodeclass:add name=fooclass")
    run_rake_task(dashboard, "nodegroup:addclass name=foogroup class=fooclass")
    run_rake_task(dashboard, "nodegroup:addclassparam name=foogroup class=fooclass param=fooparam value=foovalue")
    run_rake_task(dashboard, "nodegroup:addclassparam name=foogroup class=fooclass param=barparam value=barvalue")

    results = run_rake_task(dashboard, "nodegroup:listclasses name=foogroup")
    assert(results.include?("fooclass"), "nodegroup:addclass should have added 'fooclass'!  nodegroup:listclasses returned: #{results.join("\n")}")

    results = run_rake_task(dashboard, "nodegroup:listclassparams name=foogroup class=fooclass")
    assert(results.include?("fooparam = \"foovalue\""), "nodegroup:addclassparam should have added 'fooclass = \"fooparam\"'! nodegroup:listclassparams returned: #{results.join("\n")}")
    assert(results.include?("barparam = \"barvalue\""), "nodegroup:addclassparam should have added 'barclass = \"barparam\"'! nodegroup:listclassparams returned: #{results.join("\n")}")

    run_rake_task(dashboard, "nodegroup:delclassparam name=foogroup class=fooclass param=fooparam")
    run_rake_task(dashboard, "nodegroup:delclassparam name=foogroup class=fooclass param=barparam")

    results = run_rake_task(dashboard, "nodegroup:listclassparams name=foogroup class=fooclass")
    assert((! results.include?("fooparam = \"foovalue\"")), "nodegroup:delclassparam should have removed 'fooclass = \"fooparam\"'! nodegroup:listclassparams returned: #{results.join("\n")}")
    assert((! results.include?("barparam = \"barvalue\"")), "nodegroup:delclassparam should have removed 'barclass = \"barparam\"'! nodegroup:listclassparams returned: #{results.join("\n")}")

    run_rake_task(dashboard, "nodegroup:del name=foogroup")
    run_rake_task(dashboard, "nodeclass:del name=fooclass")
  end
end


#################################
# `node` tasks
#################################

step "test node:add, node:list, node:del rake tasks" do
  dashboard_hosts.each do |dashboard|
    run_rake_task(dashboard, "node:add name=foo.localdomain")
    results = run_rake_task(dashboard, "node:list")
    assert(results.include?("foo.localdomain"), "node:add should have added 'foo.localdomain'!  node:list returned: #{results.join("\n")}")

    run_rake_task(dashboard, "node:del name=foo.localdomain")
    results = run_rake_task(dashboard, "node:list")
    assert((! results.include?("foo.localdomain")), "node:del should have removed 'foo.localdomain'!  node:list returned: #{results.join("\n")}")
  end
end

step "test node:classes, node:listclasses rake tasks" do
  dashboard_hosts.each do |dashboard|
    run_rake_task(dashboard, "node:add name=foo.localdomain")
    run_rake_task(dashboard, "nodeclass:add name=fooclass")
    run_rake_task(dashboard, "nodeclass:add name=barclass")
    run_rake_task(dashboard, "node:classes name=foo.localdomain classes=fooclass,barclass")

    results = run_rake_task(dashboard, "node:listclasses name=foo.localdomain")
    assert(results.include?("fooclass"), "node:classes should have added 'fooclass'!  node:listclasses returned: #{results.join("\n")}")
    assert(results.include?("barclass"), "node:classes should have added 'barclass'!  node:listclasses returned: #{results.join("\n")}")

    run_rake_task(dashboard, "node:classes name=foo.localdomain classes=\"\"")

    results = run_rake_task(dashboard, "node:listclasses name=foo.localdomain")
    assert((! results.include?("fooclass")), "node:classes should have removed 'fooclass'!  node:listclasses returned: #{results.join("\n")}")
    assert((! results.include?("barclass")), "node:classes should have removed 'barclass'!  node:listclasses returned: #{results.join("\n")}")

    run_rake_task(dashboard, "node:del name=foo.localdomain")
    run_rake_task(dashboard, "nodeclass:del name=fooclass")
    run_rake_task(dashboard, "nodeclass:del name=barclass")
  end
end

step "test node:addclassparam, node:delclassparam, node:listclassparams rake tasks" do
  dashboard_hosts.each do |dashboard|
    run_rake_task(dashboard, "node:add name=foo.localdomain")
    run_rake_task(dashboard, "nodeclass:add name=fooclass")
    run_rake_task(dashboard, "node:classes name=foo.localdomain classes=fooclass")
    run_rake_task(dashboard, "node:addclassparam name=foo.localdomain class=fooclass param=fooparam value=foovalue")
    run_rake_task(dashboard, "node:addclassparam name=foo.localdomain class=fooclass param=barparam value=barvalue")

    results = run_rake_task(dashboard, "node:listclasses name=foo.localdomain")
    assert(results.include?("fooclass"), "node:classes should have added 'fooclass'!  node:listclasses returned: #{results.join("\n")}")

    results = run_rake_task(dashboard, "node:listclassparams name=foo.localdomain class=fooclass")
    assert(results.include?("fooparam = \"foovalue\""), "node:addclassparam should have added 'fooclass = \"fooparam\"'! node:listclassparams returned: #{results.join("\n")}")
    assert(results.include?("barparam = \"barvalue\""), "node:addclassparam should have added 'barclass = \"barparam\"'! node:listclassparams returned: #{results.join("\n")}")

    run_rake_task(dashboard, "node:delclassparam name=foo.localdomain class=fooclass param=fooparam")
    run_rake_task(dashboard, "node:delclassparam name=foo.localdomain class=fooclass param=barparam")

    results = run_rake_task(dashboard, "node:listclassparams name=foo.localdomain class=fooclass")
    assert((! results.include?("fooparam = \"foovalue\"")), "node:delclassparam should have removed 'fooclass = \"fooparam\"'! node:listclassparams returned: #{results.join("\n")}")
    assert((! results.include?("barparam = \"barvalue\"")), "node:delclassparam should have removed 'barclass = \"barparam\"'! node:listclassparams returned: #{results.join("\n")}")

    run_rake_task(dashboard, "node:del name=foo.localdomain")
    run_rake_task(dashboard, "nodeclass:del name=fooclass")
  end
end

step "test node:groups rake task" do
  dashboard_hosts.each do |dashboard|
    run_rake_task(dashboard, "node:add name=foo.localdomain")
    run_rake_task(dashboard, "nodegroup:add name=foogroup")
    run_rake_task(dashboard, "nodegroup:add name=bargroup")
    run_rake_task(dashboard, "node:groups name=foo.localdomain groups=foogroup,bargroup")

    results = run_rake_task(dashboard, "node:listgroups name=foo.localdomain")
    assert(results.include?("foogroup"), "node:groups should have added 'foogroup'!  node:listgroups returned: #{results.join("\n")}")
    assert(results.include?("bargroup"), "node:groups should have added 'bargroup'!  node:listgroups returned: #{results.join("\n")}")

    run_rake_task(dashboard, "node:groups name=foo.localdomain groups=\"\"")

    results = run_rake_task(dashboard, "node:listgroups name=foo.localdomain")
    assert((! results.include?("foogroup")), "node:groups should have removed 'foogroup'!  node:listgroups returned: #{results.join("\n")}")
    assert((! results.include?("bargroup")), "node:groups should have removed 'bargroup'!  node:listgroups returned: #{results.join("\n")}")

    run_rake_task(dashboard, "node:del name=foo.localdomain")
    run_rake_task(dashboard, "nodegroup:del name=foogroup")
    run_rake_task(dashboard, "nodegroup:del name=bargroup")
  end
end

#########################################################
# `node` and `nodegroup` :parameters and :variables tasks
#########################################################

["node", "nodegroup"].each do |namespace|
  ["variables", "parameters"].each do |task|

    full_task = "#{namespace}:#{task}"
    step "test #{full_task} rake tasks" do
      dashboard_hosts.each do |dashboard|

        run_rake_task(dashboard, "#{namespace}:add name=foo")
        run_rake_task(dashboard, "#{full_task} name=foo #{task}=foovar=fooval,barvar=barval,letters=a\\\\,b\\\\,c,pet=cat\\\\=socks")

        results = run_rake_task(dashboard, "#{full_task} name=foo")
        assert(results.include?("foovar=fooval"), "#{full_task} should have added 'foovar=fooval'! #{full_task} name=foo returned: #{results.join("\n")}")
        assert(results.include?("barvar=barval"), "#{full_task} should have added 'barvar=barval'! #{full_task} name=foo returned: #{results.join("\n")}")
        assert(results.include?("letters=a,b,c"), "#{full_task} should have added 'letters=a,b,c'! #{full_task} name=bar returned: #{results.join("\n")}")
        assert(results.include?("pet=cat=socks"), "#{full_task} should have added 'pet=cat=socks'! #{full_task} name=bar returned: #{results.join("\n")}")

        if task == "parameters"
          assert(results.include?("The #{namespace}:parameters task is deprecated. Please use the #{namespace}:variables task instead"),
                 "#{namespace}:parameters should have stated 'The #{namespace}:parameters task is deprecated. Please use the #{namespace}:variables task instead'! #{namespace}:parameters name=foo returned: #{results.join("\n")}")
        end

        run_rake_task(dashboard, "#{namespace}:del name=foo")
      end
    end
  end
end
