namespace :defaultgroup do
  RAKE_ROOT = File.dirname(__FILE__)

  desc 'Ensure that the default group exists'
  task :ensure_default_group => :environment do
    lockfile = File.new("#{RAKE_ROOT}/default_group.lock", File::CREAT)
    if lockfile.flock(File::LOCK_NB | File::LOCK_EX)
      default_group = NodeGroup.find_or_create_by_name('default')
      mco_group     = NodeGroup.find_or_create_by_name('mcollective')
      no_mco_group  = NodeGroup.find_or_create_by_name('no mcollective')
      mco_class     = NodeClass.find_or_create_by_name('pe_mcollective')
      account_class = NodeClass.find_or_create_by_name('pe_accounts')

      ## Add descriptions
      if not no_mco_group.description
        no_mco_group.description = <<-EOS
Add nodes to this class to prevent active management of MCollective.
This is useful for device nodes or PE agents which should not connect
to the MCollective service.
        EOS
        no_mco_group.save
      end

      if not mco_group.description
        mco_group.description = <<-EOS
Nodes are automatically added to this group unless they are a member of the 'no
mcollective' group.  This group manages the MCollective configuration on PE
agent nodes.
        EOS
        mco_group.save
      end

      # Make sure the default group does not contain the pe_mcollective class.
      group_classes = default_group.node_classes
      if group_classes.include?(mco_class)
        group_classes.delete(mco_class)
        default_group.save
      end

      # Make sure the "no mcollective" group does not contain the pe_mcollective class.
      no_mco_group_classes = no_mco_group.node_classes
      if no_mco_group_classes.include?(mco_class)
        no_mco_group_classes.delete(mco_class)
        no_mco_group.save
      end

      # Make sure the MCollective group contains the pe_mcollective class.
      mco_group_classes = mco_group.node_classes
      unless mco_group_classes.include?(mco_class)
        mco_group_classes << mco_class
        mco_group.save
      end

      name = "default"
      Rake::Task['nodegroup:add_all_nodes'].invoke(name)
      Rake::Task['nodegroup:add_mcollective_nodes'].invoke(name)

      lockfile.flock(File::LOCK_UN)
    end
  end
end
